#!/usr/bin/env python2

import numpy as np
import matplotlib.pyplot as plt
from ompl import base as ob
from ompl import geometric as og
 
def plot_points(points, specs='r'):
        x_val = [x[0] for x in points]
        y_val = [x[1] for x in points]
        plt.plot(x_val, y_val, specs)
        plt.draw()
 
def plot_edges(edges):
        ed = np.asarray(edges)
        print(ed)
        for edge in ed:
            plt.plot([edge[0,0], edge[1,0]],[edge[0,1], edge[1,1]],'g',linewidth=0.6)

        plt.show()
    
def isStateValid(state):
        #method for collision detection checking
        return (state[0] >= state[1])
 
def plan(start, goal):
        #set dimensions of the problem to be solved
        stateSpace = ob.RealVectorStateSpace()  #set state space
        stateSpace.addDimension(0.0, 10.0) #set width
        stateSpace.addDimension(0.0, 10.0) #set height
 
        #create a simple setup object
        task = og.SimpleSetup(stateSpace)
 
        #set methods for collision detection and resolution of the checking 
        task.setStateValidityChecker(ob.StateValidityCheckerFn(isStateValid))
        task.getSpaceInformation().setStateValidityCheckingResolution(0.001)
 
        #setting start and goal positions
        start_pose = ob.State(task.getStateSpace())   
        goal_pose = ob.State(task.getStateSpace())
        start_pose()[0] = start[0]
        start_pose()[1] = start[1]
        goal_pose()[0] = goal[0]
        goal_pose()[1] = goal[1]
        task.setStartAndGoalStates(start_pose, goal_pose)
 
        #setting particular planner from the supported planners
        info = task.getSpaceInformation()
        planner = og.PRM(info)   # RRT, RRTConnect, FMT, ...
        task.setPlanner(planner)
 
        #find the solution
        solution = task.solve()
 
        #simplify the solution
        #if solution:
        #       task.simplifySolution()
 
        #retrieve found path
        plan = task.getSolutionPath()
 
        #extract path and plot it
        path = []
        for i in range(plan.getStateCount()):
                path.append((plan.getState(i)[0], plan.getState(i)[1]))
 
        plot_points(path,'r')
        plt.show()

        #retrieve planner data
        plannerdata = ob.PlannerData(info)
        planner.getPlannerData(plannerdata)
 
        #getting vertices
        nv = plannerdata.numVertices()
        g = []
        for i in range(0, nv):
                vert = plannerdata.getVertex(i)
                g.append((vert.getState()[0], vert.getState()[1]))
 
        #getting edges
        e = []
        for i in range(0, nv):
                for j in range(0, nv):
                        if plannerdata.edgeExists(i, j):
                                e.append((g[i], g[j]))
        #show edges
        plot_edges(e)
        plt.show()
 
 
if __name__ == "__main__":
        plan([0, 0], [10, 10])
